#!/bin/sh

cover -delete

OLD_HARNESS_PERL_SWITCHES=$HARNESS_PERL_SWITCHES
HARNESS_PERL_SWITCHES=-MDevel::Cover
export HARNESS_PERL_SWITCHES

make test

HARNESS_PERL_SWITCHES=$OLD_HARNESS_PERL_SWITCHES
cover
